@@ -5,7 +5,7 @@
 
     Several HTML builders.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -849,13 +849,17 @@ class StandaloneHTMLBuilder(Builder):
         self.finish_tasks.add_task(self.dump_inventory)
 
     def dump_inventory(self):
+        def safe_name(string):
+            return re.sub("\s+", " ", string)
+
         self.info(bold('dumping object inventory... '), nonl=True)
         with open(path.join(self.outdir, INVENTORY_FILENAME), 'wb') as f:
             f.write((u'# Sphinx inventory version 2\n'
                      u'# Project: %s\n'
                      u'# Version: %s\n'
                      u'# The remainder of this file is compressed using zlib.\n'
-                     % (self.config.project, self.config.version)).encode('utf-8'))
+                     % (safe_name(self.config.project),
+                        safe_name(self.config.version))).encode('utf-8'))
             compressor = zlib.compressobj(9)
             for domainname, domain in sorted(self.env.domains.items()):
                 for name, dispname, type, docname, anchor, prio in \
@@ -964,7 +968,8 @@ class SingleFileHTMLBuilder(StandaloneHTMLBuilder):
         if 'includehidden' not in kwds:
             kwds['includehidden'] = False
         toctree = self.env.get_toctree_for(docname, self, collapse, **kwds)
-        self.fix_refuris(toctree)
+        if toctree is not None:
+            self.fix_refuris(toctree)
         return self.render_partial(toctree)['fragment']
 
     def assemble_doctree(self):
