@@ -5,7 +5,7 @@
 
     docutils writers handling Sphinx' custom nodes.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -131,7 +131,7 @@ class HTMLTranslator(BaseTranslator):
         pass
 
     def visit_desc_returns(self, node):
-        self.body.append(' &rarr; ')
+        self.body.append(' &#x2192; ')
 
     def depart_desc_returns(self, node):
         pass
@@ -343,6 +343,27 @@ class HTMLTranslator(BaseTranslator):
         if isinstance(node.parent, nodes.table):
             self.body.append('<span class="caption-text">')
 
+    def depart_title(self, node):
+        close_tag = self.context[-1]
+        if (self.permalink_text and self.builder.add_permalinks and
+           node.parent.hasattr('ids') and node.parent['ids']):
+            # add permalink anchor
+            if close_tag.startswith('</h'):
+                self.add_permalink_ref(node.parent, _('Permalink to this headline'))
+            elif close_tag.startswith('</a></h'):
+                self.body.append(u'</a><a class="headerlink" href="#%s" ' %
+                                 node.parent['ids'][0] +
+                                 u'title="%s">%s' % (
+                                     _('Permalink to this headline'),
+                                     self.permalink_text))
+            elif isinstance(node.parent, nodes.table):
+                self.body.append('</span>')
+                self.add_permalink_ref(node.parent, _('Permalink to this table'))
+        elif isinstance(node.parent, nodes.table):
+            self.body.append('</span>')
+
+        BaseTranslator.depart_title(self, node)
+
     # overwritten
     def visit_literal_block(self, node):
         if node.rawsource != node.astext():
@@ -433,7 +454,7 @@ class HTMLTranslator(BaseTranslator):
                 self.body.append(self.starttag(production, 'strong', ''))
                 self.body.append(lastname + '</strong> ::= ')
             elif lastname is not None:
-                self.body.append('%s     ' % (' '*len(lastname)))
+                self.body.append('%s     ' % (' ' * len(lastname)))
             production.walkabout(self)
             self.body.append('\n')
         self.body.append('</pre>\n')
@@ -500,7 +521,7 @@ class HTMLTranslator(BaseTranslator):
                                          self.builder.images[olduri])
 
         uri = node['uri']
-        if uri.lower().endswith('svg') or uri.lower().endswith('svgz'):
+        if uri.lower().endswith(('svg', 'svgz')):
             atts = {'src': uri}
             if 'width' in node:
                 atts['width'] = node['width']
@@ -532,6 +553,13 @@ class HTMLTranslator(BaseTranslator):
                         node['height'] = str(size[1])
         BaseTranslator.visit_image(self, node)
 
+    # overwritten
+    def depart_image(self, node):
+        if node['uri'].lower().endswith(('svg', 'svgz')):
+            self.body.append(self.context.pop())
+        else:
+            BaseTranslator.depart_image(self, node)
+
     def visit_toctree(self, node):
         # this only happens when formatting a toc from env.tocs -- in this
         # case we don't want to include the subtree
@@ -567,6 +595,10 @@ class HTMLTranslator(BaseTranslator):
     def depart_hlistcol(self, node):
         self.body.append('</td>')
 
+    def visit_option_group(self, node):
+        BaseTranslator.visit_option_group(self, node)
+        self.context[-2] = self.context[-2].replace('&nbsp;', '&#160;')
+
     def bulk_text_processor(self, text):
         return text
 
@@ -586,7 +618,7 @@ class HTMLTranslator(BaseTranslator):
                     self.body.append(token)
                 else:
                     # protect runs of multiple spaces; the last one can wrap
-                    self.body.append('&#160;' * (len(token)-1) + ' ')
+                    self.body.append('&#160;' * (len(token) - 1) + ' ')
         else:
             if self.in_mailto and self.settings.cloak_email_addresses:
                 encoded = self.cloak_email(encoded)
@@ -670,24 +702,6 @@ class HTMLTranslator(BaseTranslator):
     def depart_abbreviation(self, node):
         self.body.append('</abbr>')
 
-    # overwritten (but not changed) to keep pair of visit/depart_term
-    def visit_term(self, node):
-        self.body.append(self.starttag(node, 'dt', ''))
-
-    # overwritten to add '</dt>' in 'depart_term' state.
-    def depart_term(self, node):
-        self.body.append('</dt>\n')
-
-    # overwritten to do not add '</dt>' in 'visit_definition' state.
-    def visit_definition(self, node):
-        self.generate_targets_for_listing(node)
-        self.body.append(self.starttag(node, 'dd', ''))
-        self.set_first_last(node)
-
-    # overwritten (but not changed) to keep pair of visit/depart_definition
-    def depart_definition(self, node):
-        self.body.append('</dd>\n')
-
     def visit_termsep(self, node):
         warnings.warn('sphinx.addnodes.termsep will be removed at Sphinx-1.6. '
                       'This warning is displayed because some Sphinx extension '
@@ -702,25 +716,6 @@ class HTMLTranslator(BaseTranslator):
     def depart_manpage(self, node):
         return self.depart_literal_emphasis(node)
 
-    def depart_title(self, node):
-        close_tag = self.context[-1]
-        if (self.permalink_text and self.builder.add_permalinks and
-           node.parent.hasattr('ids') and node.parent['ids']):
-            # add permalink anchor
-            if close_tag.startswith('</h'):
-                self.add_permalink_ref(node.parent, _('Permalink to this headline'))
-            elif close_tag.startswith('</a></h'):
-                self.body.append(u'</a><a class="headerlink" href="#%s" ' %
-                                 node.parent['ids'][0] +
-                                 u'title="%s">%s' % (
-                                     _('Permalink to this headline'),
-                                     self.permalink_text))
-            elif isinstance(node.parent, nodes.table):
-                self.body.append('</span>')
-                self.add_permalink_ref(node.parent, _('Permalink to this table'))
-
-        BaseTranslator.depart_title(self, node)
-
     # overwritten to add even/odd classes
 
     def visit_table(self, node):
@@ -736,6 +731,11 @@ class HTMLTranslator(BaseTranslator):
         self.body.append(self.starttag(node, 'tr', ''))
         node.column = 0
 
+    def visit_entry(self, node):
+        BaseTranslator.visit_entry(self, node)
+        if self.body[-1] == '&nbsp;':
+            self.body[-1] = '&#160;'
+
     def visit_field_list(self, node):
         self._fieldlist_row_index = 0
         return BaseTranslator.visit_field_list(self, node)
@@ -748,6 +748,12 @@ class HTMLTranslator(BaseTranslator):
             node['classes'].append('field-odd')
         self.body.append(self.starttag(node, 'tr', '', CLASS='field'))
 
+    def visit_field_name(self, node):
+        context_count = len(self.context)
+        BaseTranslator.visit_field_name(self, node)
+        if context_count != len(self.context):
+            self.context[-1] = self.context[-1].replace('&nbsp;', '&#160;')
+
     def visit_math(self, node, math_env=''):
         self.builder.warn('using "math" markup without a Sphinx math extension '
                           'active, please use one of the math extensions '
