@@ -8,7 +8,7 @@
     Much of this code is adapted from Dave Kuhlman's "docpy" writer from his
     docutils sandbox.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -50,15 +50,14 @@ DEFAULT_SETTINGS = {
     'latex_engine':    'pdflatex',
     'papersize':       'letterpaper',
     'pointsize':       '10pt',
-    'pxunit':          '49336sp',
+    'pxunit':          '.75bp',
     'classoptions':    '',
     'extraclassoptions': '',
     'maxlistdepth':    '',
     'sphinxpkgoptions':     '',
     'sphinxsetup':     '',
     'passoptionstopackages': '',
-    'geometry':       ('\\usepackage[margin=1in,marginparwidth=0.5in]'
-                       '{geometry}'),
+    'geometry':        '\\usepackage{geometry}',
     'inputenc':        '',
     'utf8extra':       '',
     'cmappkg':         '\\usepackage{cmap}',
@@ -85,14 +84,14 @@ DEFAULT_SETTINGS = {
     'release':         '',
     'author':          '',
     'logo':            '',
-    'releasename':     'Release',
+    'releasename':     '',
     'makeindex':       '\\makeindex',
     'shorthandoff':    '',
     'maketitle':       '\\maketitle',
     'tableofcontents': '\\sphinxtableofcontents',
     'atendofbody':     '',
     'printindex':      '\\printindex',
-    'transition':      '\n\n\\bigskip\\hrule{}\\bigskip\n\n',
+    'transition':      '\n\n\\bigskip\\hrule\\bigskip\n\n',
     'figure_align':    'htbp',
     'tocdepth':        '',
     'secnumdepth':     '',
@@ -103,8 +102,9 @@ ADDITIONAL_SETTINGS = {
     'pdflatex': {
         'inputenc':     '\\usepackage[utf8]{inputenc}',
         'utf8extra':   ('\\ifdefined\\DeclareUnicodeCharacter\n'
+                        ' \\ifdefined\\DeclareUnicodeCharacterAsOptional\\else\n'
                         '  \\DeclareUnicodeCharacter{00A0}{\\nobreakspace}\n'
-                        '\\fi'),
+                        '\\fi\\fi'),
     },
     'xelatex': {
         'latex_engine': 'xelatex',
@@ -122,6 +122,7 @@ ADDITIONAL_SETTINGS = {
     },
     'platex': {
         'latex_engine': 'platex',
+        'geometry':     '\\usepackage[dvipdfm]{geometry}',
     },
 }
 
@@ -173,9 +174,10 @@ class ExtBabel(Babel):
         if shortlang in ('de', 'ngerman', 'sl', 'slovene', 'pt', 'portuges',
                          'es', 'spanish', 'nl', 'dutch', 'pl', 'polish', 'it',
                          'italian'):
-            return '\\if\\catcode`\\"\\active\\shorthandoff{"}\\fi'
+            return '\\ifnum\\catcode`\\"=\\active\\shorthandoff{"}\\fi'
         elif shortlang in ('tr', 'turkish'):
-            return '\\if\\catcode`\\=\\active\\shorthandoff{=}\\fi'
+            # memo: if ever Sphinx starts supporting 'Latin', do as for Turkish
+            return '\\ifnum\\catcode`\\=\\string=\\active\\shorthandoff{=}\\fi'
         return ''
 
     def uses_cyrillic(self):
@@ -355,6 +357,7 @@ class LaTeXTranslator(nodes.NodeVisitor):
         self.this_is_the_title = 1
         self.literal_whitespace = 0
         self.no_contractions = 0
+        self.in_parsed_literal = 0
         self.compact_list = 0
         self.first_param = 0
         self.remember_multirow = {}
@@ -381,12 +384,15 @@ class LaTeXTranslator(nodes.NodeVisitor):
         self.elements.update({
             'wrapperclass': self.format_docclass(document.settings.docclass),
             # if empty, the title is set to the first section title
-            'title':        document.settings.title,
-            'release':      builder.config.release,
-            'author':       document.settings.author,
-            'releasename':  _('Release'),
+            'title':        document.settings.title,    # treat as a raw LaTeX code
+            'release':      self.encode(builder.config.release),
+            'author':       document.settings.author,   # treat as a raw LaTeX code
             'indexname':    _('Index'),
         })
+        if not self.elements['releasename']:
+            self.elements.update({
+                'releasename':  _('Release'),
+            })
         if not builder.config.latex_keep_old_macro_names:
             self.elements['sphinxpkgoptions'] = 'dontkeepoldnames'
         if document.settings.docclass == 'howto':
@@ -611,34 +617,34 @@ class LaTeXTranslator(nodes.NodeVisitor):
         figure = self.builder.config.numfig_format['figure'].split('%s', 1)
         if len(figure) == 1:
             ret.append('\\def\\fnum@figure{%s}\n' %
-                       escape_abbr(text_type(figure[0]).translate(tex_escape_map)))
+                       text_type(figure[0]).strip().translate(tex_escape_map))
         else:
-            definition = escape_abbr(text_type(figure[0]).translate(tex_escape_map))
+            definition = text_type(figure[0]).strip().translate(tex_escape_map)
             ret.append(self.babel_renewcommand('\\figurename', definition))
             if figure[1]:
                 ret.append('\\makeatletter\n')
                 ret.append('\\def\\fnum@figure{\\figurename\\thefigure%s}\n' %
-                           escape_abbr(text_type(figure[1]).translate(tex_escape_map)))
+                           text_type(figure[1]).strip().translate(tex_escape_map))
                 ret.append('\\makeatother\n')
 
         table = self.builder.config.numfig_format['table'].split('%s', 1)
         if len(table) == 1:
             ret.append('\\def\\fnum@table{%s}\n' %
-                       escape_abbr(text_type(table[0]).translate(tex_escape_map)))
+                       text_type(table[0]).strip().translate(tex_escape_map))
         else:
-            definition = escape_abbr(text_type(table[0]).translate(tex_escape_map))
+            definition = text_type(table[0]).strip().translate(tex_escape_map)
             ret.append(self.babel_renewcommand('\\tablename', definition))
             if table[1]:
                 ret.append('\\makeatletter\n')
                 ret.append('\\def\\fnum@table{\\tablename\\thetable%s}\n' %
-                           escape_abbr(text_type(table[1]).translate(tex_escape_map)))
+                           text_type(table[1]).strip().translate(tex_escape_map))
                 ret.append('\\makeatother\n')
 
         codeblock = self.builder.config.numfig_format['code-block'].split('%s', 1)
         if len(codeblock) == 1:
             pass  # FIXME
         else:
-            definition = escape_abbr(text_type(codeblock[0]).translate(tex_escape_map))
+            definition = text_type(codeblock[0]).strip().translate(tex_escape_map)
             ret.append(self.babel_renewcommand('\\literalblockname', definition))
             if codeblock[1]:
                 pass  # FIXME
@@ -1029,14 +1035,20 @@ class LaTeXTranslator(nodes.NodeVisitor):
             self.body.append('%%\n\\begin{footnotetext}[%s]'
                              '\\sphinxAtStartFootnote\n' % node['number'])
         else:
-            self.body.append('%%\n\\begin{footnote}[%s]'
-                             '\\sphinxAtStartFootnote\n' % node['number'])
+            if self.in_parsed_literal:
+                self.body.append('\\begin{footnote}[%s]' % node['number'])
+            else:
+                self.body.append('%%\n\\begin{footnote}[%s]' % node['number'])
+            self.body.append('\\sphinxAtStartFootnote\n')
 
     def depart_collected_footnote(self, node):
         if 'footnotetext' in node:
             self.body.append('%\n\\end{footnotetext}')
         else:
-            self.body.append('%\n\\end{footnote}')
+            if self.in_parsed_literal:
+                self.body.append('\\end{footnote}')
+            else:
+                self.body.append('%\n\\end{footnote}')
         self.in_footnote -= 1
 
     def visit_label(self, node):
@@ -1118,12 +1130,14 @@ class LaTeXTranslator(nodes.NodeVisitor):
             self.body.extend(self.tableheaders)
             self.body.append('\\endfirsthead\n\n')
             self.body.append('\\multicolumn{%s}{c}%%\n' % self.table.colcount)
-            self.body.append(r'{{\tablecontinued{\tablename\ \thetable{} -- %s}}} \\'
+            self.body.append(r'{\makebox[0pt]{\tablecontinued{\tablename\ '
+                             r'\thetable{} -- %s}}}\\'
                              % _('continued from previous page'))
             self.body.append('\n\\hline\n')
             self.body.extend(self.tableheaders)
             self.body.append('\\endhead\n\n')
-            self.body.append(r'\hline \multicolumn{%s}{|r|}{{\tablecontinued{%s}}} \\ \hline'
+            self.body.append(r'\hline \multicolumn{%s}{|r|}{\makebox[0pt][r]'
+                             r'{\tablecontinued{%s}}}\\\hline'
                              % (self.table.colcount,
                                 _('Continued on next page')))
             self.body.append('\n\\endfoot\n\n')
@@ -1433,6 +1447,9 @@ class LaTeXTranslator(nodes.NodeVisitor):
         attrs = node.attributes
         pre = []                        # in reverse order
         post = []
+        if self.in_parsed_literal:
+            pre = ['\\begingroup\\sphinxunactivateextrasandspace\\relax ']
+            post = ['\\endgroup ']
         include_graphics_options = []
         is_inline = self.is_inline(node)
         if 'width' in attrs:
@@ -1641,7 +1658,7 @@ class LaTeXTranslator(nodes.NodeVisitor):
         parindex = node.parent.index(node)
         try:
             try:
-                next = node.parent[parindex+1]
+                next = node.parent[parindex + 1]
             except IndexError:
                 # last node in parent, look at next after parent
                 # (for section of equal level) if it exists
@@ -1700,14 +1717,14 @@ class LaTeXTranslator(nodes.NodeVisitor):
                 elif type == 'pair':
                     p1, p2 = [self.encode(x) for x in split_into(2, 'pair', string)]
                     self.body.append(r'\index{%s!%s%s}\index{%s!%s%s}' %
-                                     (p1, p2, m,  p2, p1, m))
+                                     (p1, p2, m, p2, p1, m))
                 elif type == 'triple':
                     p1, p2, p3 = [self.encode(x)
                                   for x in split_into(3, 'triple', string)]
                     self.body.append(
                         r'\index{%s!%s %s%s}\index{%s!%s, %s%s}'
                         r'\index{%s!%s %s%s}' %
-                        (p1, p2, p3, m,  p2, p3, p1, m,  p3, p1, p2, m))
+                        (p1, p2, p3, m, p2, p3, p1, m, p3, p1, p2, m))
                 elif type == 'see':
                     p1, p2 = [self.encode(x) for x in split_into(2, 'see', string)]
                     self.body.append(r'\index{%s|see{%s}}' % (p1, p2))
@@ -1739,12 +1756,12 @@ class LaTeXTranslator(nodes.NodeVisitor):
         elif uri.startswith(URI_SCHEMES):
             if len(node) == 1 and uri == node[0]:
                 if node.get('nolinkurl'):
-                    self.body.append('\\nolinkurl{%s}' % self.encode_uri(uri))
+                    self.body.append('\\sphinxnolinkurl{%s}' % self.encode_uri(uri))
                 else:
-                    self.body.append('\\url{%s}' % self.encode_uri(uri))
+                    self.body.append('\\sphinxurl{%s}' % self.encode_uri(uri))
                 raise nodes.SkipNode
             else:
-                self.body.append('\\href{%s}{' % self.encode_uri(uri))
+                self.body.append('\\sphinxhref{%s}{' % self.encode_uri(uri))
                 self.context.append('}')
         elif uri.startswith('#'):
             # references to labels in the same document
@@ -1927,7 +1944,8 @@ class LaTeXTranslator(nodes.NodeVisitor):
     def visit_literal_block(self, node):
         if node.rawsource != node.astext():
             # most probably a parsed-literal block -- don't highlight
-            self.body.append('\\begin{alltt}\n')
+            self.in_parsed_literal += 1
+            self.body.append('\\begin{sphinxalltt}\n')
         else:
             ids = ''
             for id in self.pop_hyperlink_ids('code-block'):
@@ -1977,16 +1995,18 @@ class LaTeXTranslator(nodes.NodeVisitor):
                                         '\\begin{sphinxVerbatim}')
             # get consistent trailer
             hlcode = hlcode.rstrip()[:-14]  # strip \end{Verbatim}
-            self.body.append('\n' + hlcode + '\\end{sphinxVerbatim')
             if self.table and not self.in_footnote:
-                self.body.append('intable')
-            self.body.append('}\n')
+                hlcode += '\\end{sphinxVerbatimintable}'
+            else:
+                hlcode += '\\end{sphinxVerbatim}'
+            self.body.append('\n' + hlcode + '\n')
             if ids:
                 self.body.append('\\let\\sphinxLiteralBlockLabel\\empty\n')
             raise nodes.SkipNode
 
     def depart_literal_block(self, node):
-        self.body.append('\n\\end{alltt}\n')
+        self.body.append('\n\\end{sphinxalltt}\n')
+        self.in_parsed_literal -= 1
     visit_doctest_block = visit_literal_block
     depart_doctest_block = depart_literal_block
 
@@ -2189,8 +2209,10 @@ class LaTeXTranslator(nodes.NodeVisitor):
 
     def visit_Text(self, node):
         text = self.encode(node.astext())
-        if not self.no_contractions:
-            text = educate_quotes_latex(text)
+        if not self.no_contractions and not self.in_parsed_literal:
+            text = educate_quotes_latex(text,
+                                        dquotes=("\\sphinxquotedblleft{}",
+                                                 "\\sphinxquotedblright{}"))
         self.body.append(text)
 
     def depart_Text(self, node):
